/*
 * Copyright (C) 2019 favdb
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package storybook.edit;

import assistant.Assistant;
import i18n.I18N;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.beans.PropertyChangeEvent;
import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;
import org.miginfocom.swing.MigLayout;
import resources.icons.ICONS;
import resources.icons.IconUtil;
import shef.ShefEditor;
import storybook.App;
import storybook.Const;
import storybook.exim.importer.ImportDocument;
import storybook.model.EntityUtil;
import storybook.model.SceneUtil;
import storybook.model.book.Book;
import storybook.model.book.Book.TYPE;
import storybook.model.hbn.entity.AbstractEntity;
import storybook.model.hbn.entity.Chapter;
import storybook.model.hbn.entity.Endnote;
import storybook.model.hbn.entity.Item;
import storybook.model.hbn.entity.Location;
import storybook.model.hbn.entity.Person;
import storybook.model.hbn.entity.Plot;
import storybook.model.hbn.entity.Scene;
import storybook.model.hbn.entity.Strand;
import storybook.toolkit.LOG;
import storybook.toolkit.Markdown;
import storybook.toolkit.SbDuration;
import storybook.toolkit.ShefUtil;
import storybook.toolkit.StringUtil;
import storybook.toolkit.file.IOUtil;
import storybook.toolkit.file.TempUtil;
import storybook.toolkit.file.XEditorFile;
import storybook.toolkit.html.Html;
import storybook.toolkit.net.Net;
import storybook.toolkit.spell.SpellChecker;
import storybook.toolkit.swing.FontUtil;
import storybook.toolkit.swing.SwingUtil;
import storybook.toolkit.swing.js.JSDateChooser;
import storybook.toolkit.swing.js.JSLabel;
import storybook.ui.MIG;
import storybook.ui.Ui;
import storybook.ui.combobox.EntityCombo;
import storybook.ui.combobox.EntityComboEntity;
import storybook.ui.dialog.ListEndnotesDlg;

/**
 * panel editor for Scene
 *
 * @author favdb
 */
public class EditScene extends AbstractEditor implements CaretListener, ItemListener {

	private static final String TT = "EditScene",
			INITIALES = I18N.getMsg("duration.initiales"),
			BT_EXTERNAL = "btExternal",
			BT_IMPORT = "import",
			BT_STAGE = "btStage",
			RB_DATENONE = "date.none",
			RB_DATEFIX = "date.fixed",
			RB_DATEREL = "date.relative";

	private JTextField tfNumber, tfDuration, fRelTime, xfile;
	private JComboBox cbChapter, cbStrand, cbStage, cbRelScene, cbStatus, cbNarrator;
	private JSDateChooser dcDate;
	private CheckBoxPanel lLocations, lItems, lPersons, lPlots, lStrands;
	private JCheckBox ckInformative;
	private JRadioButton rbDatenone, rbAbs, rbRel;
	private ShefEditor htTexte;
	private JPanel pDate, pRel;
	private Markdown mkTexte;
	private EditScenario pScenario;
	private Scene scene;
	private JButton btImport, btExternal;
	private Intensity pIntensity;
	private List<Endnote> endnotes;
	private JButton btEndnotes;

	public EditScene(Editor m, AbstractEntity e) {
		super(m, e, "111");
		if (e == null) {
			LOG.err(TT + " : entity is null");
			return;
		}
		initAll();
	}

	public Scene getScene() {
		return scene;
	}

	/**
	 * initialize the upper datas
	 */
	@Override
	public void initUpper() {
		scene = (Scene) entity;
		panel.setLayout(new MigLayout(MIG.get("ins 1", MIG.WRAP, MIG.HIDEMODE2), "[][grow][]"));
		if (scene.getId() == -1) {
			scene.setChapter(mainFrame.chapterGetLast());
			scene.setDate(mainFrame.sceneGetLastDate());
		}
		JPanel panel1 = new JPanel(new MigLayout(MIG.get("ins 2", MIG.WRAP, MIG.HIDEMODE2), "[][grow]"));
		panel1.setMinimumSize(MAXIMUM_SIZE);
		cbStatus = Ui.initStatus(panel1, scene.getStatus(), MANDATORY);

		Ui.addLabel(panel1, "number", false, MANDATORY);
		tfNumber = Ui.getStringField("number", 4, scene.getSceneno().toString(), !INFO);
		panel1.add(tfNumber, "split 2");

		if (scene.getSceneno() < 1) {
			tfNumber.setText("+");
		}

		ckInformative = Ui.initCheckBox(panel1, "ckInformative", "informative", scene.getInformative(), !MANDATORY, "skip 1");
		ckInformative.setToolTipText(I18N.getMsg("informative.tip"));

		if (scene.getName().isEmpty() && scene.getChapter() == null) {
			//this is a new scene so init chapter with last chapter used
			scene.setChapter(mainFrame.chapterGetLast());
		}
		cbChapter = Ui.getComboBox(panel1, this, TYPE.CHAPTER, scene.getChapter(), null, !MANDATORY, NEW, EMPTY);
		cbChapter.setMaximumSize(new Dimension(512, cbChapter.getPreferredSize().height));
		JLabel spacer = new JLabel("    ");
		spacer.setMaximumSize(new Dimension(64, cbChapter.getPreferredSize().height));

		if (EntityUtil.findEntities(mainFrame, TYPE.STRAND).size() > 1) {
			cbStrand = Ui.getComboBox(panel1, this, TYPE.STRAND, scene.getStrand(), null, MANDATORY, NEW, EMPTY);
			if (scene.getStrand() == null) {
				cbStrand.setSelectedIndex(1);
			}
			cbStrand.addItemListener(this);
		}

		if (!EntityUtil.findEntities(mainFrame, TYPE.PERSON).isEmpty()) {
			cbNarrator = Ui.initComboBox(panel1, this, "scene.narrator", TYPE.PERSON,
					scene.getNarrator(), null, !MANDATORY, NEW, EMPTY);
		}

		//date
		JPanel gpDate = new JPanel(new MigLayout(MIG.get(MIG.HIDEMODE3, MIG.INS0)));
		gpDate.setBorder(BorderFactory.createTitledBorder(I18N.getMsg("date")));
		rbDatenone = Ui.initRadioButton(gpDate, RB_DATENONE, false, !MANDATORY);
		rbAbs = Ui.initRadioButton(gpDate, RB_DATEFIX, false, !MANDATORY);
		rbRel = Ui.initRadioButton(gpDate, RB_DATEREL, false, !MANDATORY, MIG.WRAP);
		//date fixe
		pDate = new JPanel(new MigLayout());
		dcDate = initSbDate(pDate, "date", scene.getScenets(), !MANDATORY, 0);
		gpDate.add(pDate, MIG.SPAN);
		//relatif à une scène
		pRel = new JPanel(new MigLayout());
		cbRelScene = new EntityCombo(mainFrame, TYPE.SCENE, scene.getRelativesceneid(), scene, "10");
		pRel.add(new JSLabel(I18N.getColonMsg("scene.relative")));
		pRel.add(cbRelScene);
		//décalage relatif en durée sous la forme yy-MM-dd_hh:mm:ss
		String t = "";
		if (!scene.getRelativetime().isEmpty()) {
			SbDuration dur = new SbDuration(scene.getRelativetime());
			t = dur.toText(INITIALES);
		}
		fRelTime = Ui.initStringField(pRel, "scene.relativetime", 16, t, !MANDATORY, !INFO);
		fRelTime.setToolTipText(I18N.getMsg("duration.format") + "\n" + I18N.getMsg("duration.use"));
		gpDate.add(pRel, MIG.SPAN);
		panel1.add(gpDate, MIG.get("skip 1", MIG.SPAN));
		ButtonGroup bg = new ButtonGroup();
		bg.add(rbDatenone);
		bg.add(rbAbs);
		bg.add(rbRel);
		rbDatenone.addActionListener(this);
		rbAbs.addActionListener(this);
		rbRel.addActionListener(this);
		setGroupDate();
		String tDuration = "";
		if (!scene.getDuration().isEmpty() || !scene.getDuration().equals("00-00-00_00:00:00")) {
			SbDuration dur = new SbDuration(scene.getDuration());
			tDuration = dur.toText(INITIALES);
		}
		tfDuration = Ui.initStringField(panel1, "duration", 16, tDuration, !MANDATORY, !INFO);
		tfDuration.setToolTipText(I18N.getMsg("duration.format") + "\n" + I18N.getMsg("duration.use"));
		//stage
		if (book == null) {
			LOG.err(TT + " book is null, may crash");
		}
		if (book.info == null) {
			LOG.err(TT + " book.info is null, may crash");
		}
		if (book.info.scenarioGet()) {
			cbStage = Ui.initComboBox(panel1, this, "scenario.stage",
					Assistant.getListOf("stage"),
					scene.getScenariostage(),
					!MANDATORY, !NEW, EMPTY);
		}
		pIntensity = new Intensity(Intensity.RADIO, scene.getIntensity());
		JLabel lbIntensity = new JLabel(I18N.getColonMsg("intensity"));
		lbIntensity.setFont(FontUtil.getBoldFont(lbIntensity.getFont()));
		panel1.add(lbIntensity, MIG.RIGHT);
		panel1.add(pIntensity, "span");
		// end common
		pScenario = new EditScenario(this);
		if (book.isMarkdown()) {
			mkTexte = new Markdown(TT, "text/plain", scene.getText());
			mkTexte.setView(Markdown.VIEW.TEXT);
			mkTexte.setDimension();
			mkTexte.setPreferredSize(MINIMUM_SIZE);
			tab1.add(mkTexte, I18N.getMsg("scene.summary"));
		} else {
			JPanel txtPanel = new JPanel(new MigLayout(MIG.get(MIG.FILL, MIG.INS0)));
			htTexte = new ShefEditor("lang_all, allow, colored", scene.getSummary());
			endnotes = Endnote.find(mainFrame, 0, scene);
			htTexte.setPreferredSize(MINIMUM_SIZE);
			ShefUtil.setToFileButton(mainFrame, htTexte);
			ShefUtil.setFromFileButton(mainFrame, htTexte);
			ShefUtil.setInternalLink(mainFrame, htTexte);
			addExternalSpell();
			JScrollPane scroller = new JScrollPane(htTexte);
			SwingUtil.setUnitIncrement(scroller);
			scroller.setPreferredSize(SwingUtil.getScreenSize());
			txtPanel.add(scroller, MIG.GROW);
			initEndnote(txtPanel);
			tab1.add(txtPanel, I18N.getMsg("scene.summary"));
		}
		JPanel nt = new JPanel(new MigLayout(MIG.get(MIG.FILL, MIG.WRAP), "[grow][grow][grow]"));
		lPersons = Ui.initCbList(nt, mainFrame, TYPE.PERSON, scene.getPersons(), !MANDATORY, null);
		lLocations = Ui.initCbList(nt, mainFrame, TYPE.LOCATION, scene.getLocations(), !MANDATORY, null);
		lItems = Ui.initCbList(nt, mainFrame, TYPE.ITEM, scene.getItems(), !MANDATORY, null);
		lPlots = Ui.initCbList(nt, mainFrame, TYPE.PLOT, scene.getPlots(), !MANDATORY, null);
		lStrands = Ui.initCbList(nt, mainFrame, TYPE.STRAND, scene.getStrands(), !MANDATORY, null);
		lStrands.removeEntity(scene.getStrand());
		JScrollPane scroll = new JScrollPane(nt);
		tab1.add(scroll, I18N.getMsg("links"));

		if (book.isScenario()) {
			pScenario.setScene(scene);
			JTabbedPane tb0 = new JTabbedPane();
			tb0.add(I18N.getMsg("scenario"), pScenario);
			JScrollPane s = new JScrollPane(panel1);
			tb0.add(I18N.getMsg("scene"), s);
			panel.add(tb0, MIG.get(MIG.GROWX, MIG.SPAN));
		} else {
			JScrollPane sc = new JScrollPane(panel1);
			sc.setPreferredSize(SwingUtil.getScreenSize());
			sc.setBorder(BorderFactory.createEmptyBorder());
			panel.add(sc, MIG.get(MIG.GROW, MIG.SPAN));
		}
	}

	/**
	 * initialize the bottom datas
	 */
	@Override
	public void initBottom() {
		JPanel pxFile = new JPanel(new MigLayout(MIG.get(MIG.FLOWX, MIG.INS0)));
		String fname = IOUtil.fileToAbsolute(mainFrame.getH2File().getPath(), scene.getOdf());
		xfile = initFileChooser(pxFile, "scene.xfile", fname);
		File file = new File(fname);
		if (!file.exists()) {
			xfile.setForeground(Color.red);
		}
		Icon xicon = IconUtil.getIconSmall(ICONS.K.EDIT);
		switch (book.getParamEditor().getName().toLowerCase()) {
			case "ms word":
				xicon = IconUtil.getIconSmall(ICONS.K.MSWORD);
				break;
			case "libreoffice":
				xicon = IconUtil.getIconSmall(ICONS.K.WRITER);
				break;
		}
		btImport = SwingUtil.createButton(BT_IMPORT, "", "import", true, this);
		btImport.setIcon(xicon);
		btImport.setToolTipText(I18N.getMsg("import.text"));
		pxFile.add(btImport);
		btExternal = SwingUtil.createButton(BT_EXTERNAL, "", "", true, this);
		btExternal.setText(book.getParamEditor().getName());
		btExternal.setIcon(xicon);
		btExternal.setToolTipText(I18N.getMsg("xeditor.launching") + " " + book.getParamEditor().getName());
		btExternal.addActionListener((ActionEvent evt) -> {
			if (xfile.getText().isEmpty()) {
				return;
			}
			String name = xfile.getText();
			XEditorFile.createFile(mainFrame, name);
			name = XEditorFile.launchExternal(name);
			xfile.setText(name);
		});
		pxFile.add(btExternal);
		add(pxFile, MIG.get(MIG.SPAN, MIG.GROWX));
		setExportImport();
		xfile.addCaretListener(this);
		if (!book.isUseXeditor()) {
			SwingUtil.setEnable(pxFile, false);
		}
	}

	/**
	 * allow or disallow the import button
	 */
	private void setExportImport() {
		if (btImport != null) {
			if (!xfile.getText().isEmpty()) {
				File f = new File(IOUtil.fileToAbsolute(mainFrame.getH2File().getPath(), xfile.getText()));
				btImport.setEnabled(f.exists());
				xfile.setForeground(!f.exists() ? Color.red : App.getDefaultForeground());
			} else {
				btImport.setEnabled(false);
			}
		}
		if (btExternal != null) {
			btExternal.setEnabled(!xfile.getText().isEmpty());
		}
		this.editor.setBtExternal(xfile.getText());
	}

	/**
	 * get the external editor file name
	 *
	 * @return
	 */
	public JTextField getXfile() {
		return xfile;
	}

	/**
	 * get the text of the Scene
	 *
	 * @return
	 */
	public String getText() {
		return (book.isMarkdown() ? mkTexte.getText() : checkImages(htTexte.getText()));
	}

	/**
	 * verifier for all datas
	 *
	 * @return
	 */
	@Override
	public boolean verifier() {
		resetError();
		@SuppressWarnings("unchecked")
		List<Scene> scenes = EntityUtil.findEntities(mainFrame, Book.TYPE.SCENE);
		if (tfNumber.getText().equals("+")) {
			tfNumber.setText(Scene.getNextNumber(scenes).toString());
		}
		if (tfNumber.getText().isEmpty()) {
			errorMsg(tfNumber, Const.ERROR_MISSING);
		} else if (!StringUtil.isNumeric(tfNumber.getText())) {
			errorMsg(tfNumber, Const.ERROR_NOTNUMERIC);
		} else {
			int n = Integer.parseInt(tfNumber.getText());
			Scene c = Scene.findNumber(scenes, n);
			if (c != null
					&& (cbChapter.getSelectedItem() != null && c.getChapter() != null
					&& c.getChapter().equals(cbChapter.getSelectedItem()))
					&& !Objects.equals(c.getId(), entity.getId())) {
				errorMsg(tfNumber, Const.ERROR_EXISTS);
			}
		}
		if (cbStatus.getSelectedIndex() < 0) {
			errorMsg(cbStatus, Const.ERROR_MISSING);
		}
		if (rbRel.isSelected() && !fRelTime.getText().isEmpty()) {
			String str = SbDuration.check(fRelTime.getText(), INITIALES);
			if (!str.isEmpty()) {
				errorString(fRelTime, str);
			}
			if (cbRelScene.getSelectedItem() == null) {
				errorMsg(cbRelScene, Const.ERROR_MISSING);
			}
		}
		if (!tfDuration.getText().isEmpty()) {
			String str = SbDuration.check(tfDuration.getText(), INITIALES);
			if (!str.isEmpty()) {
				errorString(tfDuration, str);
			}
		}
		String text;
		JComponent comp;
		if (book.isMarkdown()) {
			text = mkTexte.getText();
			comp = mkTexte;
		} else {
			text = Html.fileToRel(htTexte.getText(), mainFrame.getH2File().getPath());
			comp = htTexte;
		}
		if (text.length() > 32768) {
			errorMsg(comp, Const.ERROR_EXCEED);
		}
		return (msgError.isEmpty());
	}

	/**
	 * get the name of the Scene
	 *
	 * @return
	 */
	public String getTfName() {
		return tfName.getText();
	}

	/**
	 * apply the changes
	 */
	@Override
	public void apply() {
		//App.trace(TT + ".apply()");
		scene.setTitle(tfName.getText());
		scene.setInformative(ckInformative.isSelected());
		if (cbChapter.getSelectedIndex() > 0) {
			scene.setChapter((Chapter) cbChapter.getSelectedItem());
		} else {
			scene.setChapter(null);
		}
		if (tfNumber.getText().equals("+")) {
			scene.setSceneno(SceneUtil.getNextNumber(mainFrame, scene.getChapter()));
		} else {
			scene.setSceneno(Integer.parseInt(tfNumber.getText()));
		}
		if (cbStrand != null && cbStrand.getSelectedIndex() > 0) {
			scene.setStrand((Strand) cbStrand.getSelectedItem());
		} else if (scene.getStrand() == null) {
			scene.setStrand((Strand) EntityUtil.findEntities(mainFrame, TYPE.STRAND).get(0));
		}
		scene.setStatus(cbStatus.getSelectedIndex());
		scene.setIntensity(pIntensity.getValue());
		if (tfDuration.getText().isEmpty()) {
			scene.setDuration("");
		} else {
			SbDuration dur = SbDuration.getFromText(tfDuration.getText(), INITIALES);
			scene.setDuration(dur.toString());
		}
		if (cbNarrator != null && cbNarrator.getSelectedIndex() > 0) {
			scene.setNarrator((Person) cbNarrator.getSelectedItem());
		} else {
			scene.setNarrator(null);
		}
		if (book.isUseXeditor()) {
			scene.setOdf(xfile.getText());
		} else {
			scene.setOdf("");
		}
		if (rbDatenone.isSelected()) {
			scene.setDate(null);
			scene.setRelativetime("");
			scene.setRelativesceneid(-1L);
		} else if (rbAbs.isSelected()) {
			scene.setDate(dcDate.getDate());
			scene.setRelativetime("");
			scene.setRelativesceneid(-1L);
		} else {
			scene.setDate(null);
			SbDuration d = SbDuration.getFromText(fRelTime.getText(), INITIALES);
			scene.setRelativetime(d.toString());
			scene.setRelativesceneid(((EntityComboEntity) cbRelScene.getSelectedItem()).getId());
		}
		if (book.isScenario()) {
			pScenario.getScenarioData(scene);
		}
		String text = getText();
		if (!book.isMarkdown()) {
			text = applyEndnotes();
		}
		scene.setSummary(text);
		if (cbStage != null && cbStage.getSelectedIndex() != -1) {
			scene.setScenario_stage(cbStage.getSelectedIndex());
		} else {
			scene.setScenario_stage(0);
		}
		scene.setPersons(getPersons());
		scene.setLocations(getLocations());
		scene.setItems(getItems());
		scene.setPlots(getPlots());
		scene.setStrands(getStrands());
		if (scene.getChapter() != null) {
			mainFrame.chapterSetLast(scene.getChapter());
		}
		mainFrame.sceneSetLastDate(scene.getDate());
		super.apply();
	}

	public Markdown getMkTexte() {
		return mkTexte;
	}

	/**
	 * perform a model property change
	 *
	 * @param evt
	 */
	@Override
	public void modelPropertyChange(PropertyChangeEvent evt) {
		// empty
	}

	/**
	 * set the date in a group data
	 */
	private void setGroupDate() {
		if (scene.hasRelativescene()) {
			rbRel.setSelected(true);
		} else if (scene.hasScenets()) {
			rbAbs.setSelected(true);
		} else {
			rbDatenone.setSelected(true);
		}
		pRel.setVisible(rbRel.isSelected());
		pDate.setVisible(rbAbs.isSelected());
	}

	/**
	 * perform an action
	 *
	 * @param evt
	 */
	@Override
	public void actionPerformed(ActionEvent evt) {
		//App.traceEvent(TT+".actionPerformed", evt);
		if (evt.getSource() instanceof JButton) {
			JButton bt = (JButton) evt.getSource();
			switch (bt.getName()) {
				case BT_IMPORT:
					File f = new File(xfile.getText());
					if (f.exists()) {
						if (!(Html.htmlToText(htTexte.getText())).isEmpty()) {
							String buttonTexts[] = {
								I18N.getMsg("import.text.replace"),
								I18N.getMsg("import.text.copy"),
								I18N.getMsg("cancel")};
							int ret = JOptionPane.showOptionDialog(this,
									I18N.getMsg("import.text.exists"),
									I18N.getMsg("import"),
									JOptionPane.YES_NO_CANCEL_OPTION,
									JOptionPane.PLAIN_MESSAGE,
									null,
									buttonTexts,
									null);
							switch (ret) {
								case JOptionPane.YES_OPTION:
									//simply replace
									break;
								case JOptionPane.NO_OPTION:
									hNotes.setText(hNotes.getText() + "\n" + htTexte.getText());
									break;
								case JOptionPane.CANCEL_OPTION:
								default:
									return;
							}
						}
						ImportDocument doc = new ImportDocument(mainFrame, f);
						if (doc.openDocument()) {
							htTexte.setText(doc.getDocument().body().html());
							doc.close();
						}
					} else {
						JOptionPane.showMessageDialog(this,
								I18N.getMsg("file.not.exist", xfile.getText()),
								I18N.getMsg("file.import"),
								JOptionPane.ERROR_MESSAGE);
					}
					break;
				case BT_STAGE:
					Net.openBrowser(I18N.getMsg("assistant.stage.web"));
					break;
				default:
					break;
			}
		} else if (evt.getSource() instanceof JRadioButton) {
			JRadioButton rb = (JRadioButton) evt.getSource();
			pDate.setVisible(false);
			pRel.setVisible(false);
			switch (rb.getName()) {
				case RB_DATEFIX:
					pDate.setVisible(true);
					break;
				case RB_DATEREL:
					pRel.setVisible(true);
					break;
				default:
					break;
			}
		}
	}

	/**
	 * temporary save
	 */
	@Override
	public void tempSave() {
		if (entity instanceof Scene) {
			EntityUtil.copyEntityProperties(mainFrame, entity, scene);
			if (book.isScenario()) {
				scene.setSummary(mkTexte.getText());
			} else {
				scene.setSummary(htTexte.getText());
			}
			TempUtil.write(mainFrame, entity);
			EntityUtil.copyEntityProperties(mainFrame, scene, entity);
		}
	}

	/**
	 * set the externale editor file name
	 *
	 * @param name
	 */
	public void setXfile(String name) {
		xfile.setText(name);
	}

	@Override
	public void caretUpdate(CaretEvent e) {
		setExportImport();
	}

	/**
	 * get the selected Persons
	 *
	 * @return
	 */
	public List<Person> getPersons() {
		List<Person> ls = new ArrayList<>();
		lPersons.getSelectedEntities().forEach((e) -> {
			ls.add((Person) e);
		});
		return ls;
	}

	/**
	 * get the selected Locations
	 *
	 * @return
	 */
	public List<Location> getLocations() {
		List<Location> ls = new ArrayList<>();
		lLocations.getSelectedEntities().forEach((e) -> {
			ls.add((Location) e);
		});
		return ls;
	}

	/**
	 * get the selected Items
	 *
	 * @return
	 */
	public List<Item> getItems() {
		List<Item> ls = new ArrayList<>();
		lItems.getSelectedEntities().forEach((e) -> {
			ls.add((Item) e);
		});
		return ls;
	}

	/**
	 * get the selected Plots
	 *
	 * @return
	 */
	public List<Plot> getPlots() {
		List<Plot> ls = new ArrayList<>();
		lPlots.getSelectedEntities().forEach((e) -> {
			ls.add((Plot) e);
		});
		return ls;
	}

	/**
	 * get the selected Strands
	 *
	 * @return
	 */
	public List<Strand> getStrands() {
		List<Strand> ls = new ArrayList<>();
		lStrands.getSelectedEntities().forEach((e) -> {
			ls.add((Strand) e);
		});
		return ls;
	}

	/**
	 * get the scenario picth text
	 *
	 * @return
	 */
	public String getScenario_pitch() {
		if (pScenario != null) {
			return pScenario.getPitch();
		}
		return "";
	}

	@Override
	@SuppressWarnings("unchecked")
	public void itemStateChanged(ItemEvent e) {
		if (e.getSource() instanceof JComboBox) {
			JComboBox cb = (JComboBox) e.getSource();
			if (cb.getName().equals(TYPE.STRAND.toString())) {
				Strand st = (Strand) cbStrand.getSelectedItem();
				lStrands.removeAll();
				lStrands.reload();
				lStrands.removeEntity(st);
				List strands = scene.getStrands();
				strands.remove(st);
				lStrands.selectEntities(strands);
			}
		}
	}

	/**
	 * add external spelling checker
	 */
	private void addExternalSpell() {
		if (SpellChecker.checkForCkEditor()) {
			JButton button = Ui.initButton("spell", "", ICONS.K.LANGTOOL, "spell", e -> {
				SpellChecker spell = new SpellChecker(mainFrame);
				spell.setText(scene.getSummary());
				spell.launchExternal();
			});
			ShefUtil.addButton(mainFrame, htTexte, button);
		}
	}

	/**
	 * check all endnotes and remove if not exists in text
	 */
	private String applyEndnotes() {
		//LOG.trace(TT+".applyEndnotes()");
		String text = getText();
		for (Endnote en : endnotes) {
			if (!text.contains(Endnote.linkTo("", en))) {
				mainFrame.getBookModel().deleteEntity(en);
			}
		}
		return text;
	}

	/**
	 * initialize the endnote functions
	 *
	 * @return a JButton for activate the liste of Endnotes
	 */
	private void initEndnote(JPanel textPanel) {
		//LOG.trace(TT + ".initEndnote(textPanel)");
		// add endnote button in Shef toolbar
		htTexte.wysEditorGet().getToolBar2().addSeparator();
		JButton bten = new JButton(IconUtil.getIconSmall(ICONS.K.CHAR_ENDNOTE));
		bten.setToolTipText(I18N.getMsg("shef.endnote"));
		bten.addActionListener(e -> createEndnote(0));
		htTexte.wysEditorGet().getToolBar2().add(bten);
		// define endnotes button at bottom texte panel
		btEndnotes = Ui.initButton("endnote",
				"endnote",
				ICONS.K.SORT,
				"endnote",
				e -> ListEndnotesDlg.showDlg(this, 0));
		btEndnotes.setEnabled(!endnotes.isEmpty());
		textPanel.add(btEndnotes, MIG.NEWLINE);
	}

	private void createEndnote(int type) {
		Endnote en;
		int m = 1;
		if (type == 0) {
			for (Endnote enote : endnotes) {
				m = Math.max(m, enote.getNumber() + 1);
			}
		}
		en = new Endnote(0, scene, m);
		if (mainFrame.showEditorAsDialog(en)) {
			return;
		}
		en = Endnote.find(mainFrame, type, scene, m);
		String link = Endnote.linkTo("", en);
		htTexte.insertText(link);
	}

}
