/*
 * Copyright (C) 2022 favdb
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package storybook.toolkit.html;

import java.awt.Font;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.nio.file.Files;
import resources.MainResources;
import storybook.App;
import storybook.toolkit.LOG;
import static storybook.toolkit.html.Html.CSS_BEGIN;
import static storybook.toolkit.html.Html.CSS_END;
import static storybook.toolkit.html.Html.FONT_FAMILY;
import static storybook.toolkit.html.Html.FONT_SIZE;
import static storybook.toolkit.html.Html.FONT_STYLE;
import static storybook.toolkit.html.Html.MARGIN_BOTTOM;
import static storybook.toolkit.html.Html.MARGIN_LEFT;
import static storybook.toolkit.html.Html.MARGIN_RIGHT;
import static storybook.toolkit.html.Html.MARGIN_TOP;
import static storybook.toolkit.html.Html.PADDING_LEFT;
import static storybook.toolkit.html.Html.PADDING_RIGHT;
import static storybook.toolkit.html.Html.TEXT_ALIGN;
import storybook.toolkit.swing.LaF;

/**
 *
 * @author favdb
 */
public class CSS {

	private static final String TT = "CSS";

	private CSS() {
		//empty
	}

	/**
	 * return a CSS param
	 *
	 * @param tab
	 * @param key
	 * @param value
	 * @param unit
	 * @return
	 */
	private static String getParam(int tab, String key, int value, String unit) {
		return (getParam(tab, key, Integer.toString(value) + unit));
	}

	public static String getParam(int tab, String key, String... value) {
		StringBuilder b = new StringBuilder();
		for (int i = 0; i < tab; i++) {
			b.append("    ");
		}
		if (key.isEmpty()) {
			b.append("}\n");
			return (b.toString());
		}
		b.append(key);
		if (value != null && value.length > 0 && !value[0].isEmpty()) {
			b.append(": ");
			b.append(value[0]);
			b.append(";");
		} else if (tab == 1) {
			b.append(" {");
		}
		b.append("\n");
		return (b.toString());
	}

	/**
	 * return a default CSS style with a given Font and a given Margin
	 *
	 * @param font
	 * @param margin
	 * @param isPanel
	 * @return
	 */
	public static String getDefault(Font font, int margin, boolean isPanel) {
		StringBuilder buf = new StringBuilder();
		String unit = "pt";
		// body
		buf.append(getParam(1, "body"));
		if (font != null) {
			buf.append(getParam(2, FONT_FAMILY, font.getFamily()));
			buf.append(CSS.getParam(2, FONT_SIZE, font.getSize(), unit));
		}
		buf.append(CSS.getParam(2, PADDING_LEFT, margin, unit))
				.append(CSS.getParam(2, PADDING_RIGHT, margin, unit));
		buf.append(getParam(1, ""));
		//h1
		buf.append(getParam(1, "h1"))
				.append(CSS.getParam(2, FONT_SIZE, 140, "%"))
				.append(getParam(2, TEXT_ALIGN, "center"))
				.append(CSS.getParam(2, MARGIN_TOP, 10, unit))
				.append(CSS.getParam(2, MARGIN_BOTTOM, 1, unit));
		buf.append(getParam(1, ""));
		//h2
		buf.append(getParam(1, "h2"))
				.append(CSS.getParam(2, FONT_SIZE, 130, "%"))
				.append(CSS.getParam(2, MARGIN_TOP, 10, unit))
				.append(CSS.getParam(2, MARGIN_BOTTOM, 1, unit));
		buf.append(getParam(1, ""));
		//h3
		buf.append(getParam(1, "h3"))
				.append(CSS.getParam(2, FONT_SIZE, 120, "%"))
				.append(CSS.getParam(2, MARGIN_TOP, 6, unit))
				.append(CSS.getParam(2, MARGIN_BOTTOM, 1, unit));
		buf.append(getParam(1, ""));
		//h4
		buf.append(getParam(1, "h4"))
				.append(CSS.getParam(2, FONT_SIZE, 110, "%"))
				.append(CSS.getParam(2, MARGIN_TOP, 6, unit))
				.append(CSS.getParam(2, MARGIN_BOTTOM, 1, unit));
		buf.append(getParam(1, ""));
		//h5
		buf.append(getParam(1, "h5"))
				.append(CSS.getParam(2, FONT_SIZE, 100, "%"))
				.append(CSS.getParam(2, MARGIN_TOP, 6, unit))
				.append(CSS.getParam(2, MARGIN_BOTTOM, 1, unit));
		buf.append(getParam(1, ""));
		//h6
		buf.append(getParam(1, "h6"))
				.append(CSS.getParam(2, FONT_SIZE, 100, "%"))
				.append(CSS.getParam(2, MARGIN_TOP, 6, unit))
				.append(CSS.getParam(2, MARGIN_BOTTOM, 1, unit));
		buf.append(getParam(1, ""));
		// links
		if (isPanel && LaF.isDark()) {
			buf.append(getParam(1, "a"))
					.append(getParam(2, "color", "#e0ffff"));
			buf.append(getParam(1, ""));
		}
		// paragraphe
		buf.append(getParam(1, "p"))
				.append(CSS.getParam(2, MARGIN_TOP, 0, unit))
				.append(CSS.getParam(2, MARGIN_BOTTOM, 6, unit))
				.append(getParam(2, TEXT_ALIGN, "justify"));
		buf.append(getParam(1, ""));
		// blockquote
		buf.append(getParam(1, "blockquote"))
				.append(CSS.getParam(2, MARGIN_TOP, 0, unit))
				.append(CSS.getParam(2, MARGIN_BOTTOM, 6, unit))
				.append(CSS.getParam(2, MARGIN_RIGHT, 40, unit))
				.append(CSS.getParam(2, MARGIN_LEFT, 40, unit))
				.append(getParam(2, TEXT_ALIGN, "justify"))
				.append(getParam(2, FONT_STYLE, "italic"));
		buf.append(getParam(1, ""));
		//nav bar
		buf.append(getParam(1, "#navbar"))
				.append(getParam(2, "background-color", "lightgrey"))
				.append(getParam(2, TEXT_ALIGN, "right"));
		buf.append(getParam(1, ""));
		//ul
		buf.append(getParam(1, "ul"))
				.append(CSS.getParam(2, MARGIN_TOP, 2, unit))
				.append(CSS.getParam(2, MARGIN_BOTTOM, 2, unit))
				.append(CSS.getParam(2, MARGIN_LEFT, 15, unit));
		buf.append(getParam(1, ""));
		//ol
		buf.append(getParam(1, "ol"))
				.append(CSS.getParam(2, MARGIN_TOP, 2, unit))
				.append(CSS.getParam(2, MARGIN_BOTTOM, 2, unit))
				.append(CSS.getParam(2, MARGIN_LEFT, 15, unit));
		buf.append(getParam(1, ""));
		//table
		buf.append(getParam(1, "table tr"))
				.append(CSS.getParam(2, "margin", 0, unit))
				.append(CSS.getParam(2, "padding", 0, unit));
		buf.append(getParam(1, ""));
		buf.append(getParam(1, "td"))
				.append(CSS.getParam(2, MARGIN_RIGHT, 1, unit))
				.append(CSS.getParam(2, "padding", 1, unit));
		buf.append(getParam(1, ""));
		buf.append(getParam(1, ".endnote"))
				.append(CSS.getParam(2, "font-size: .83em; vertical-align: super", 0, unit));
		buf.append(getParam(1, ""));
		return buf.toString();
	}

	/**
	 * get CSS for scenario
	 *
	 * @param force: get standard CSS and force font-family, size and margin-top and bottom left and
	 * right margin will be 1cm. If not forced, then get the CSS ressource and force font-family and
	 * font size to the default font mono
	 *
	 * @return String containing the CSS code
	 */
	public static String forScenario(boolean force) {
		String buf = "", unit = "pt";
		Font font = App.getInstance().fontGetMono();
		if (force) {
			buf = "body {"
					+ FONT_FAMILY + ": " + font.getFontName() + ";"
					+ FONT_SIZE + ": " + font.getSize() + unit
					+ "}\n"
					+ "p {"
					+ MARGIN_LEFT + ": 0.5cm;"
					+ MARGIN_RIGHT + ": 0.5cm;"
					+ MARGIN_TOP + ": " + font.getSize() / 2 + unit + ";"
					+ MARGIN_BOTTOM + ": " + font.getSize() / 2 + unit + ";"
					+ "}\n";
			return (buf);
		}
		File file = new File(MainResources.class.getResource("css/scenario.css").getFile());
		try {
			buf = new String(Files.readAllBytes(file.toPath()));
		} catch (IOException ex) {
		}
		String str = buf.replace("monospace", font.getName());
		str = str.replace(FONT_SIZE + ": 12pt", FONT_SIZE + ": " + Integer.toString(font.getSize()) + unit);
		str = str.replace("8pt", Integer.toString(font.getSize() / 2) + unit);
		return (str);
	}

	/**
	 * get the Scenario CSS from a file
	 *
	 * @param cssFile
	 * @return
	 */
	public static String fromFile(String cssFile) {
		String buf = "";
		File file = new File(cssFile);
		try {
			buf = new String(Files.readAllBytes(file.toPath()));
		} catch (IOException ex) {
		}
		return (buf);
	}

	/**
	 * return a CSS using a file with a given Font
	 *
	 * @param nf
	 * @param font
	 * @return
	 */
	public static String fromFile(String nf, Font font) {
		StringBuilder b = new StringBuilder();
		b.append(CSS_BEGIN);
		if (nf != null && !nf.isEmpty()) {
			File f = new File(nf);
			if (f.exists()) {
				try (BufferedReader br = new BufferedReader(new FileReader(f))) {
					String ligne;
					while ((ligne = br.readLine()) != null) {
						b.append(ligne).append("\n");
					}
					br.close();
				} catch (IOException e) {
					LOG.err(TT + ".getCss(nf, font) error reading CSS");
				} finally {

				}
			}
		} else {
			b.append(getDefault(font, 0, false));
		}
		b.append(CSS_END);
		return (b.toString());
	}

	/**
	 * return a HEAD with CSS STYLE for a given Font
	 *
	 * @param font
	 * @return
	 */
	public static String getHeadWithCss(Font font) {
		String buf = Html.HEAD
				+ Html.CSS_BEGIN
				+ CSS.getDefault(font, 0, false)
				+ Html.CSS_END
				+ Html.HEAD_END;
		return buf;
	}

	/**
	 * return a HEAD with default CSS STYLE
	 *
	 * @return
	 */
	public static String getHeadWithCss() {
		return getHeadWithCss(App.getInstance().fontGetDefault());
	}

}
