/*
 * Copyright (C) 2021 favdb
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package storybook.ui;

import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import javax.swing.JButton;
import javax.swing.JToolBar;
import javax.swing.SwingUtilities;
import resources.icons.ICONS;
import storybook.Pref;
import storybook.model.hbn.entity.Chapter;
import storybook.model.hbn.entity.Gender;
import storybook.model.hbn.entity.Idea;
import storybook.model.hbn.entity.Item;
import storybook.model.hbn.entity.Itemlink;
import storybook.model.hbn.entity.Location;
import storybook.model.hbn.entity.Memo;
import storybook.model.hbn.entity.Part;
import storybook.model.hbn.entity.Person;
import storybook.model.hbn.entity.Relationship;
import storybook.model.hbn.entity.Scene;
import storybook.model.hbn.entity.Strand;
import storybook.model.hbn.entity.Tag;
import storybook.model.hbn.entity.Taglink;
import storybook.toolkit.swing.SwingUtil;
import storybook.ui.dialog.ToolBarDlg;

/**
 *
 * @author favdb
 */
public class Toolbar extends JToolBar implements MouseListener {

	private final static String TT = "Toolbar";

	private final MainFrame mainFrame;
	public JButton btFileNew, btFileOpen, btFileSave,
			btIdea, btMemo,
			btNewChapter, btNewGender, btNewItem, btNewItemlink, btNewLocation,
			btNewPart, btNewPerson, btNewRelation, btNewScene, btNewStrand,
			btNewTag, btNewTaglink,
			btTabChapter, btTabGender, btTabIdea, btTabItem, btTabItemlink,
			btTabLocation, btTabMemo, btTabPart, btTabPerson, btTabRelation,
			btTabScene, btTabStrand, btTabTag, btTabTaglink,
			btViewBook, btViewChrono, btViewManage, btViewMemoria,
			btViewReading, btViewStoryboard, btViewTypist,
			btViewTimeline;

	public Toolbar(MainFrame mainFrame) {
		this.mainFrame = mainFrame;
		init();
	}

	private void init() {
		setFloatable(false);
		setName("MainToolbar");
		Separator separator = new JToolBar.Separator();
		initTbFile();
		add(separator);
		initTbNew();
		add(separator);
		initTbTab();
		add(separator);
		initTbView();
		add(separator);
		initTbMemo();
	}

	private void initTbFile() {
		//LOG.trace(TT+".initTbFile()");
		btFileNew = initButton(ICONS.K.F_NEW, "file.new",
				evt -> mainFrame.action.fileNew());
		add(btFileNew);

		btFileOpen = initButton(ICONS.K.F_OPEN, "file.open",
				evt -> mainFrame.action.fileOpen());
		add(btFileOpen);

		btFileSave = initButton(ICONS.K.F_SAVE, "file.save",
				evt -> mainFrame.action.fileSave());
		add(btFileSave);

	}

	private void initTbNew() {
		//LOG.trace(TT+".initTbNew()");
		btNewStrand = initButton(ICONS.K.ENT_STRAND, "strand.new", evt -> mainFrame.newEntity(new Strand()));
		add(btNewStrand);

		btNewPart = initButton(ICONS.K.ENT_PART, "part.new", evt -> mainFrame.newEntity(new Part()));
		add(btNewPart);

		btNewChapter = initButton(ICONS.K.ENT_CHAPTER, "chapter.new", evt -> mainFrame.newEntity(new Chapter()));
		add(btNewChapter);

		btNewScene = initButton(ICONS.K.ENT_SCENE, "scene.new", evt -> mainFrame.newEntity(new Scene()));
		add(btNewScene);

		btNewPerson = initButton(ICONS.K.ENT_PERSON, "person.new", evt -> mainFrame.newEntity(new Person()));
		add(btNewPerson);

		btNewGender = initButton(ICONS.K.ENT_GENDER, "gender.new", evt -> mainFrame.newEntity(new Gender()));
		add(btNewGender);

		btNewRelation = initButton(ICONS.K.ENT_RELATION, "relation.new", evt -> mainFrame.newEntity(new Relationship()));
		add(btNewRelation);

		btNewLocation = initButton(ICONS.K.ENT_LOCATION, "location.new", evt -> mainFrame.newEntity(new Location()));
		add(btNewLocation);

		btNewItem = initButton(ICONS.K.ENT_ITEM, "item.new", evt -> mainFrame.newEntity(new Item()));
		add(btNewItem);

		btNewItemlink = initButton(ICONS.K.ENT_ITEMLINK, "itemlink.new", evt -> mainFrame.newEntity(new Itemlink()));
		add(btNewItemlink);

		btNewTag = initButton(ICONS.K.ENT_TAG, "tag.new", evt -> mainFrame.newEntity(new Tag()));
		add(btNewTag);

		btNewTaglink = initButton(ICONS.K.ENT_TAGLINK, "taglink.new", evt -> mainFrame.newEntity(new Taglink()));
		add(btNewTaglink);

	}

	private void initTbTab() {
		//LOG.trace(TT+".initTbTab()");
		btTabStrand = initButton(ICONS.K.TABLE_STRANDS, "strands", evt -> mainFrame.showAndFocus(SbView.VIEWNAME.STRANDS));
		add(btTabStrand);

		btTabPart = initButton(ICONS.K.TABLE_PARTS, "parts", evt -> mainFrame.showAndFocus(SbView.VIEWNAME.PARTS));
		add(btTabPart);

		btTabChapter = initButton(ICONS.K.TABLE_CHAPTERS, "chapters", evt -> mainFrame.showAndFocus(SbView.VIEWNAME.CHAPTERS));
		add(btTabChapter);

		btTabScene = initButton(ICONS.K.TABLE_SCENES, "scenes", evt -> mainFrame.showAndFocus(SbView.VIEWNAME.SCENES));
		add(btTabScene);

		btTabPerson = initButton(ICONS.K.TABLE_PERSONS, "persons", evt -> mainFrame.showAndFocus(SbView.VIEWNAME.PERSONS));
		add(btTabPerson);

		btTabGender = initButton(ICONS.K.TABLE_GENDERS, "genders", evt -> mainFrame.showAndFocus(SbView.VIEWNAME.GENDERS));
		add(btTabGender);

		btTabRelation = initButton(ICONS.K.TABLE_RELATIONS, "relations", evt -> mainFrame.showAndFocus(SbView.VIEWNAME.RELATIONS));
		add(btTabRelation);

		btTabLocation = initButton(ICONS.K.TABLE_LOCATIONS, "locations", evt -> mainFrame.showAndFocus(SbView.VIEWNAME.LOCATIONS));
		add(btTabLocation);

		btTabItem = initButton(ICONS.K.TABLE_ITEMS, "items", evt -> mainFrame.showAndFocus(SbView.VIEWNAME.ITEMS));
		add(btTabItem);

		btTabItemlink = initButton(ICONS.K.TABLE_ITEMLINKS, "itemlinks", evt -> mainFrame.showAndFocus(SbView.VIEWNAME.ITEMLINKS));
		add(btTabItemlink);

		btTabTag = initButton(ICONS.K.TABLE_TAGS, "tags", evt -> mainFrame.showAndFocus(SbView.VIEWNAME.TAGS));
		add(btTabTag);

		btTabTaglink = initButton(ICONS.K.TABLE_TAGLINKS, "taglinks", evt -> mainFrame.showAndFocus(SbView.VIEWNAME.TAGLINKS));
		add(btTabTaglink);

		btTabMemo = initButton(ICONS.K.TABLE_MEMOS, "memos", evt -> mainFrame.showAndFocus(SbView.VIEWNAME.MEMOS));
		add(btTabMemo);

		btTabIdea = initButton(ICONS.K.TABLE_IDEAS, "ideas", evt -> mainFrame.showAndFocus(SbView.VIEWNAME.IDEAS));
		add(btTabIdea);

	}

	private void initTbView() {
		//LOG.trace(TT+".initTbView()");
		btViewChrono = initButton(ICONS.K.VW_CHRONO, "view.chrono", evt -> mainFrame.showAndFocus(SbView.VIEWNAME.CHRONO));
		btViewChrono.setEnabled(false);
		add(btViewChrono);

		btViewTimeline = initButton(ICONS.K.VW_TIMELINE, "view.timeline", evt -> mainFrame.showAndFocus(SbView.VIEWNAME.TIMELINE));
		add(btViewTimeline);

		btViewBook = initButton(ICONS.K.VW_BOOK, "view.book", evt -> mainFrame.showAndFocus(SbView.VIEWNAME.BOOK));
		add(btViewBook);

		btViewManage = initButton(ICONS.K.VW_MANAGE, "view.manage", evt -> mainFrame.showAndFocus(SbView.VIEWNAME.MANAGE));
		add(btViewManage);

		btViewReading = initButton(ICONS.K.VW_READING, "view.reading");
		btViewReading.addActionListener(evt -> mainFrame.showAndFocus(SbView.VIEWNAME.READING));
		add(btViewReading);

		btViewMemoria = initButton(ICONS.K.VW_MEMORIA, "view.memoria", evt -> mainFrame.showAndFocus(SbView.VIEWNAME.MEMORIA));
		add(btViewMemoria);

		btViewStoryboard = initButton(ICONS.K.STAMP, "view.storyboard", evt -> mainFrame.showAndFocus(SbView.VIEWNAME.STORYBOARD));
		add(btViewStoryboard);

		addSeparator();

		btViewTypist = initButton(ICONS.K.VW_TYPIST, "typist", evt -> mainFrame.typistActivate());
		add(btViewTypist);

	}

	private void initTbMemo() {
		//LOG.trace(TT+".initTbMemo()");
		btMemo = initButton(ICONS.K.ENT_MEMO, "memos");
		btMemo.addActionListener(evt -> mainFrame.showEditorAsDialog(new Memo(), btMemo));
		add(btMemo);

		btIdea = initButton(ICONS.K.ENT_IDEA, "ideas");
		btIdea.addActionListener(evt -> mainFrame.showEditorAsDialog(new Idea(), btIdea));
		add(btIdea);
	}

	private JButton initButton(ICONS.K icon, String tips, ActionListener... action) {
		JButton bt = SwingUtil.createButton("", icon.toString(), tips, true, action);
		bt.addMouseListener(this);
		return bt;
	}

	public void reload() {
		init();
		set();
	}

	private void set() {
		removeMouseListener(this);
		JButton[] btn = {
			btFileNew, btFileOpen, btFileSave, //3
			btNewStrand, btNewPart, btNewChapter, btNewScene,//4
			btNewPerson, btNewGender, btNewRelation,//3
			btNewLocation,//1
			btNewItem, btNewItemlink,//2
			btNewTag, btNewTaglink, //2
			btMemo, btIdea, //2
			btTabStrand, btTabPart, btTabChapter, btTabScene,//4
			btTabPerson, btTabGender, btTabRelation, //3
			btTabLocation, //1
			btTabItem, btTabItemlink,//2
			btTabTag, btTabTaglink, //2
			btTabMemo, btTabIdea,//2
			btViewChrono, btViewTimeline, btViewBook, btViewManage, btViewReading, btViewMemoria, btViewStoryboard, btViewTypist //6
		}; // total 37
		//récupération du paramétrage sous forme d'un String
		String param = mainFrame.getPref().get(Pref.KEY.TOOLBAR.toString(), Pref.KEY.TOOLBAR.getValue());
		while (param.length() < btn.length) {
			param += "1";// default allParts button are visible
		}
		if (param.length() > btn.length) {
			param = param.substring(0, btn.length);
		}
		for (int i = 0; i < param.length(); i++) {
			btn[i].setVisible((param.charAt(i) == '1'));
		}
		setFloatable(true);
		addMouseListener(this);
	}

	public void setForBlank() {
		//LOG.trace(TT+".setForBlank()");
		javax.swing.JButton button[] = {
			btFileSave,
			btViewManage,
			btNewChapter, btNewItem, btNewLocation, btNewPerson, btNewScene, btNewTag,
			btTabChapter, btTabItem, btTabItemlink, btTabLocation, btTabPerson, btTabRelation,
			btTabScene, btTabTag, btTabTaglink,
			btViewTypist, btViewBook, btViewChrono, btViewMemoria, btViewReading, btIdea, btMemo
		};
		for (javax.swing.JButton bt : button) {
			bt.setVisible(false);
		}
	}

	public void enableSave(boolean b) {
		//LOG.trace(TT+".enableSave()");
		if (btFileSave != null) {
			btFileSave.setEnabled(b);
		}
	}

	@Override
	public void mouseClicked(MouseEvent evt) {
		//LOG.trace(TT+".mouseClicked(evt=" + evt.toString());
		if (SwingUtilities.isRightMouseButton(evt)) {
			ToolBarDlg dlg = new ToolBarDlg(mainFrame);
			dlg.setVisible(true);
			if (!dlg.isCanceled()) {
				set();
			}
			evt.consume();
		}
	}

	@Override
	public void mousePressed(MouseEvent e) {
		// empty
	}

	@Override
	public void mouseReleased(MouseEvent e) {
		// empty
	}

	@Override
	public void mouseEntered(MouseEvent e) {
		// empty
	}

	@Override
	public void mouseExited(MouseEvent e) {
		// empty
	}

	public static String getDefault() {
		return "1"//file.new
				+ "1"//file.open
				+ "1"//file.save
				+ "0"//strand.new
				+ "0"//part.new
				+ "1"//chapter.new
				+ "1"//scene.new
				+ "1"//person.new
				+ "0"//gender.new
				+ "0"//relation.new
				+ "1"//location.new
				+ "1"//item.new
				+ "0"//temlink.new
				+ "0"//tag.new
				+ "0"//taglink.new
				+ "1"//memo.new
				+ "1"//idea.new
				+ "1"//manage_strands
				+ "0"//manage_parts
				+ "1"//manage_chapters
				+ "1"//manage_scenes
				+ "1"//manage_persons
				+ "0"//manage_genders
				+ "0"//manage_relations
				+ "1"//manage_locations
				+ "1"//manage_items
				+ "0"//manage_itemlinks
				+ "0"//manage_tags
				+ "0"//manage_taglinks
				+ "0"//manage_memos
				+ "0"//manage_ideas
				+ "1"//chrono
				+ "0"//timeline
				+ "1"//book
				+ "1"//manage_view
				+ "1"//reading
				+ "1"//memoria
				+ "0"//storyboard
				+ "1"//typist
				;
	}

}
