/*
 * Copyright (C) 2020 favdb
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package storybook.ui.dialog.preferences;

import java.awt.event.ActionEvent;
import java.beans.PropertyChangeEvent;
import javax.swing.ButtonGroup;
import javax.swing.JLabel;
import javax.swing.JRadioButton;
import org.miginfocom.swing.MigLayout;
import storybook.App;
import storybook.Pref;
import i18n.I18N;
import storybook.toolkit.swing.FontUtil;
import storybook.ui.MIG;
import storybook.ui.panel.AbstractPanel;

/**
 *
 * @author favdb
 */
public class PrefExport extends AbstractPanel {

	private final PreferencesDlg caller;
	private Pref pref;
	private JLabel lbExportParam;
	private JRadioButton rbTxt;
	private JRadioButton rbCsv;
	private JRadioButton rbXml;
	private JRadioButton rbHtml;

	public PrefExport(PreferencesDlg dlg) {
		super(dlg.getMainFrame());
		this.caller = dlg;
		initAll();
	}

	@Override
	public void init() {
		pref = mainFrame.getPref();
	}

	@Override
	public void initUi() {
		setLayout(new MigLayout(MIG.WRAP, "[10%][]"));
		lbExportParam = new JLabel(I18N.getMsg("preferences.export.parameters"));
		lbExportParam.setFont(FontUtil.getItalicFont(App.getInstance().fontGetDefault()));
		add(lbExportParam, MIG.SPAN);
		rbTxt = new JRadioButton("txt");
		add(rbTxt, MIG.SKIP);
		rbCsv = new JRadioButton("csv");
		add(rbCsv, MIG.SKIP);
		rbXml = new JRadioButton("xml");
		add(rbXml, MIG.SKIP);
		rbHtml = new JRadioButton("html");
		add(rbHtml, MIG.SKIP);
		switch (pref.getString(Pref.KEY.EXP_FORMAT, "xml")) {
			case "txt":
				rbTxt.setSelected(true);
				break;
			case "csv":
				rbCsv.setSelected(true);
				break;
			case "html":
				rbHtml.setSelected(true);
				break;
			default:
				rbXml.setSelected(true);
				break;
		}
		ButtonGroup bgExport = new ButtonGroup();
		bgExport.add(rbCsv);
		bgExport.add(rbTxt);
		bgExport.add(rbHtml);
		bgExport.add(rbXml);
	}

	@Override
	public void modelPropertyChange(PropertyChangeEvent evt) {
		// empty
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		// empty
	}

	public void apply() {
		String export = "xml";
		if (rbTxt.isSelected()) {
			export = "txt";
		} else if (rbCsv.isSelected()) {
			export = "csv";
		} else if (rbHtml.isSelected()) {
			export = "html";
		} else if (rbXml.isSelected()) {
			export = "xml";
		}
		pref.setString(Pref.KEY.EXP_FORMAT, export);
	}

	public void refreshUi() {
		lbExportParam.setText(I18N.getMsg("preferences.export.parameters"));
	}

}
